import { NgZone } from '@angular/core';
import { ApolloClient, QueryOptions, MutationOptions, ApolloQueryResult, SubscriptionOptions, ApolloClientOptions, FetchResult } from '@apollo/client/core';
import { Observable } from 'rxjs';
import { QueryRef } from './QueryRef';
import { WatchQueryOptions, ExtraSubscriptionOptions, R, NamedOptions, Flags } from './types';
export declare class ApolloBase<TCacheShape = any> {
    protected ngZone: NgZone;
    protected flags?: Flags;
    protected _client?: ApolloClient<TCacheShape>;
    private useInitialLoading;
    constructor(ngZone: NgZone, flags?: Flags, _client?: ApolloClient<TCacheShape>);
    watchQuery<T, V = R>(options: WatchQueryOptions<V>): QueryRef<T, V>;
    query<T, V = R>(options: QueryOptions<V>): Observable<ApolloQueryResult<T>>;
    mutate<T, V = R>(options: MutationOptions<T, V>): Observable<FetchResult<T>>;
    subscribe<T, V = R>(options: SubscriptionOptions<V>, extra?: ExtraSubscriptionOptions): Observable<FetchResult<T>>;
    /**
     * Get an access to an instance of ApolloClient
     * @deprecated use `apollo.client` instead
     */
    getClient(): ApolloClient<TCacheShape>;
    /**
     * Set a new instance of ApolloClient
     * Remember to clean up the store before setting a new client.
     *
     * @deprecated use `apollo.client = client` instead
     * @param client ApolloClient instance
     */
    setClient(client: ApolloClient<TCacheShape>): void;
    /**
     * Get an access to an instance of ApolloClient
     */
    /**
    * Set a new instance of ApolloClient
    * Remember to clean up the store before setting a new client.
    *
    * @param client ApolloClient instance
    */
    client: ApolloClient<TCacheShape>;
    private readonly ensureClient;
    private checkInstance;
}
export declare class Apollo extends ApolloBase<any> {
    private _ngZone;
    private map;
    constructor(_ngZone: NgZone, apolloOptions?: ApolloClientOptions<any>, apolloNamedOptions?: NamedOptions, flags?: Flags);
    /**
     * Create an instance of ApolloClient
     * @param options Options required to create ApolloClient
     * @param name client's name
     */
    create<TCacheShape>(options: ApolloClientOptions<TCacheShape>, name?: string): void;
    /**
     * Use a default ApolloClient
     */
    default(): ApolloBase<any>;
    /**
     * Use a named ApolloClient
     * @param name client's name
     */
    use(name: string): ApolloBase<any>;
    /**
     * Create a default ApolloClient, same as `apollo.create(options)`
     * @param options ApolloClient's options
     */
    createDefault<TCacheShape>(options: ApolloClientOptions<TCacheShape>): void;
    /**
     * Create a named ApolloClient, same as `apollo.create(options, name)`
     * @param name client's name
     * @param options ApolloClient's options
     */
    createNamed<TCacheShape>(name: string, options: ApolloClientOptions<TCacheShape>): void;
    /**
     * Remember to clean up the store before removing a client
     * @param name client's name
     */
    removeClient(name?: string): void;
}
