"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const tasks_1 = require("@angular-devkit/schematics/tasks");
const ts = require("typescript");
const utils_1 = require("../../utils");
const index_1 = require("../../install/index");
function default_1() {
    return schematics_1.chain([migrateImports, migrateTsConfig, migrateDependencies]);
}
exports.default = default_1;
function migrateDependencies() {
    return (tree, context) => {
        const packageJsonPath = 'package.json';
        const packageJson = utils_1.getJsonFile(tree, packageJsonPath);
        packageJson.dependencies = packageJson.dependencies || {};
        for (const dependency in index_1.dependenciesMap) {
            if (index_1.dependenciesMap.hasOwnProperty(dependency)) {
                const version = index_1.dependenciesMap[dependency];
                packageJson.dependencies[dependency] = version;
            }
        }
        delete packageJson.dependencies['apollo-client'];
        delete packageJson.dependencies['apollo-link'];
        delete packageJson.dependencies['apollo-cache-inmemory'];
        delete packageJson.dependencies['graphql-tag'];
        context.logger.info('Removing apollo-client dependency');
        context.logger.info('Removing apollo-link dependency');
        context.logger.info('Removing apollo-cache-inmemory dependency');
        context.logger.info('Removing graphql-tag dependency');
        // save the changed file
        tree.overwrite(packageJsonPath, JSON.stringify(packageJson, null, 2));
        // schedule `npm install`
        context.addTask(new tasks_1.NodePackageInstallTask());
        return tree;
    };
}
function migrateTsConfig(tree) {
    return __awaiter(this, void 0, void 0, function* () {
        const tsconfigPath = 'tsconfig.json';
        const tsconfig = utils_1.getJsonFile(tree, tsconfigPath);
        const compilerOptions = tsconfig.compilerOptions;
        if (compilerOptions) {
            compilerOptions.allowSyntheticDefaultImports = true;
            tree.overwrite(tsconfigPath, JSON.stringify(tsconfig, null, 2));
        }
    });
}
exports.migrateTsConfig = migrateTsConfig;
function migrateImports(tree) {
    return __awaiter(this, void 0, void 0, function* () {
        tree.visit(path => {
            if (path.includes('node_modules') || !path.endsWith('.ts')) {
                return;
            }
            const sourceFile = ts.createSourceFile(path, tree.read(path).toString(), ts.ScriptTarget.Latest, true);
            const imports = [];
            const recorder = tree.beginUpdate(path);
            sourceFile.statements.forEach(statement => {
                if (ts.isImportDeclaration(statement) &&
                    ts.isStringLiteral(statement.moduleSpecifier)) {
                    const nodeText = statement.moduleSpecifier.getText(sourceFile);
                    const modulePath = statement.moduleSpecifier
                        .getText(sourceFile)
                        .substr(1, nodeText.length - 2);
                    if (['apollo-cache-inmemory', 'apollo-client', 'apollo-link', 'apollo-angular'].includes(modulePath)) {
                        if (statement.importClause.namedBindings) {
                            statement.importClause.namedBindings.forEachChild(named => {
                                if (ts.isImportSpecifier(named)) {
                                    const name = named.propertyName && typeof named.propertyName !== 'undefined'
                                        ? named.propertyName.escapedText.toString()
                                        : named.name.escapedText.toString();
                                    imports.push({
                                        name,
                                        alias: name === named.name.escapedText.toString() ? undefined : name,
                                    });
                                }
                            });
                        }
                        recorder.remove(statement.getStart(), statement.getWidth());
                    }
                    if (modulePath === 'graphql-tag') {
                        imports.push({
                            name: 'gql',
                            alias: statement.importClause.name.escapedText.toString() === 'gql'
                                ? undefined
                                : statement.importClause.name.escapedText.toString(),
                        });
                        recorder.remove(statement.getStart(), statement.getWidth());
                    }
                }
            });
            if (imports.length) {
                const props = imports
                    .map(im => (im.alias ? `${im.name} as ${im.alias}` : im.name))
                    .join(', ');
                recorder.insertLeft(sourceFile.getStart(), `import {${props}} from 'apollo-angular';\n`);
                tree.commitUpdate(recorder);
            }
        });
    });
}
exports.migrateImports = migrateImports;
//# sourceMappingURL=index.js.map